//GSEvents.js -- Main event handling and bot code
//Written by Buffslave II of Darktide
//Part of GarageSale Bot, under LGPL license


events = new Object();

function GSInstallEventHooks()
{
	events = new Object();
	events.OnAddToInventory = GS_OnAddToInventory;
	events.OnDeathSelf = GS_OnDeathSelf;
	events.OnTell = GS_OnTell;
	events.OnTradeAccept = GS_OnTradeAccept;
	events.OnTradeAdd = GS_OnTradeAdd;
	events.OnTradeEnd = GS_OnTradeEnd;
	events.OnTradeReset = GS_OnTradeReset;
	events.OnTradeStart = GS_OnTradeStart;

	skapi.AddHandler(evidOnAddToInventory, events);
	skapi.AddHandler(evidOnDeathSelf, events);
	skapi.AddHandler(evidOnTell, events);
	skapi.AddHandler(evidOnTradeAccept, events);
	skapi.AddHandler(evidOnTradeAdd, events);
	skapi.AddHandler(evidOnTradeEnd, events);
	skapi.AddHandler(evidOnTradeReset, events);
	skapi.AddHandler(evidOnTradeStart, events);
	
}

function GS_OnAddToInventory(aco)
{
	GSMessage("Inventory changed, will reindex salvage when next customer begins trade.");
	Self.reindex = true;
}

function GS_OnDeathSelf(acoKiller, szMsg)
{
	Self.state = STATE_INACTIVE;
	Self.cart.clear();
}

function GS_OnTell(acoSender, acoReceiver, szMsg)
{
	//Received a /t from someone other than our trading partner
	if(Self.partner == null || acoSender.oid != Self.partner.oid)
	{
		//Only respond to "location"
		if(GSIsLocationEnabled() && szMsg == "location")
		{
			var loc = GSGetLocation();
			
			if(loc != null)
				skapi.Tell(acoSender.szName, GSGetLocation());
		}
		
		return;
	}
	
	var tokens = szMsg.split(" ");
	var cmd = CMD_INVALID;
	switch(tokens[0])
	{
		case "add": //add <nr> <type> [min,max]|<num>+
		
			if(Self.puttingInWindow)
			{
				GSTellPartner("Please wait until all salvage is added.");
				return;
			}
		
			//Make sure we are still displaying salvage
			if(Self.state != STATE_DISPLAYING)
			{
				GSTellPartner("You can't add salvage now, you are checking out! Clear the trade window if you want to add more.");
				return;
			}
			if(tokens.length < 3)
			{
				GSTellPartner("You are missing stuff after 'add'. Use 'help add' for more information.");
				return;
			}
			
			if(tokens.length > 5)
			{
				GSTellPartner("You have too many things after 'add'. Use 'help add' for more information.");
				return;
			}
			
		
			var nr = parseInt(tokens[1]); // number of bags
			var type = tokens[2];
			var work;
			var workmin, workmax;
			
			//Invalid?
			if(nr <= 0)
			{
				GSTellPartner("The first thing after 'add' should be the number of bags. Use 'help add' for more information.");
				return;
			}
			
			//Does 3rd parameter have no digits? If so then it is the second word (e.g. "Hide" in "Gromnie Hide")
			if(tokens.length >= 4 && tokens[3].match(/[0-9]/) == null)
			{
				type = type + " " + tokens[3];
				
				if(tokens.length == 4)
				{
					work = "1+";
				}
				else
					work = tokens[4];
			}
			else
			{
				if(tokens.length == 3)
					work = "1+";
				else
					work = tokens[3];
			}
			
			//Parse the workmanship string
			var bounds = GSParseWork(work);
			
			//Failed to parse?
			if(bounds == null)
			{
				GSTellPartner("The workmanship bounds are invalid. Use 'help add' for more information.");
				return;
			}
			workmin = bounds.work_min;
			workmax = bounds.work_max;
						
			//Any salvage available of this type?
			var salvage = GSSalvageManager.getSalvage(type);
			if(salvage == null || salvage.totalAvailable() == 0)
			{
				GSTellPartner("No '"+type+"' is available. Use 'available' to see what salvage is available.");
				return;
			}
			
			//Can we fulfill this request?
			if(salvage.hasAvailable(workmin, workmax, nr) == false)
			{
				GSTellPartner("Not enough " +type+" salvage between ["+workmin+","+workmax+"] to satisfy your request. Please try again later.");
				return;
			}
			
			//Attempt to add more items to shopping cart
			//Since this can fail, we do not modify the shopping cart until the end
			//where we add the entire array all at once.
			var to_add = new Array();
			var remaining = nr;
			
			//We'll try every workmanship in range (unless we are done, e.g. remaining == 0)
			for(var i=workmin; i<=workmax && remaining > 0; i++)
			{
				//Get a list of OIDs for this workmanship
				oids = salvage.getOIDs(i);
					
				//Check each item in the list (unless we are done, e.g. remaining == 0)
				for(var k=0; k<oids.length && remaining > 0; k++)
				{
					//If the item isn't in our shopping cart
					if(Self.cart.containsOID(oids[k]) == false)
					{
						to_add.push(oids[k]);
						remaining--;
					}
				}
			}

			//Not able to find additional bags?
			if(remaining != 0)
			{
				GSTellPartner("Unable to add an additional " + nr + " bag(s) -- " + (nr - remaining) + " additional bag(s) are available.");
				return;
			}
			
			//Add items to shopping cart while looking up price.
			for(var k=0; k<to_add.length; k++)
			{
				var item = skapi.AcoFromOid(to_add[k]);
				var wk = Math.floor(item.workmanship);
				
				Self.cart.addItem(item, GSGetPrice(type, wk));
			}
			
			GSTellPartner("Added! Your total is now "+Self.cart.getTotal()+" point(s). Tell me \"checkout\" to add the salvage to the window and pay.");
			
			break;
			
		case "price": //price <type> (optional: <work>)
		
			if(tokens.length < 2)
			{
				GSTellPartner("You need to specify what type of salvage you want to get the price of.");
				return;
			}
			
			var type = tokens[1];
			var work = null;
			
			//Does 2nd parameter have no digits? If so then it is the second word (e.g. "Hide" in "Gromnie Hide")
			if(tokens.length > 2)
			{
				if(tokens[2].match(/[0-9]/) == null)
				{
					type = type + " " + tokens[2];
					work = tokens[3];
				}
				else
					work = tokens[2];
			}
			
			if(work == null)
				work = "1+";
			
			var salvage = GSSalvageManager.getSalvage(type);	
			if(salvage == null || salvage.totalAvailable() == 0)
			{
				GSTellPartner("None available. Please try back later, or check the spelling!");
				return;
			}

			
			//Parse the workmanship string
			var bounds = GSParseWork(work);
			
			//Failed to parse?
			if(bounds == null)
			{
				GSTellPartner("The workmanship bounds are invalid. Use 'help price' for more information.");
				return;
			}
			workmin = bounds.work_min;
			workmax = bounds.work_max;			

			GSTellPartner(GSGetPriceString(type, workmin, workmax));
			break;
			
//		case "list":
//			GSTellPartner("This function has not yet been implemented.");
//			break;
			
		case "total":
			GSTellPartner("Your total is currently "+Self.cart.getTotal()+" point(s). Tell me \"checkout\" to finish.");
			break;
			
		case "checkout":
			if(Self.state != STATE_DISPLAYING)
				GSTellPartner("You are already checking out.");
			else
			{
				var total = Self.cart.getTotal();
				if(total == 0)
				{
					GSTellPartner("You haven't added anything yet! Tell me 'help' to get started.");
					return;
				}
				

			
				skapi.TradeReset();
				Self.cart.addToTrade();
				GSTellPartner("Please place payment of " + Self.cart.getTotal() + " points in the trade window.");
				Self.state = STATE_SUBTOTALING;
			}
			break;
			
		case "available": //available <type>
			if(tokens.length > 3)
			{
				GSTellPartner("You have too many things after 'available'. Use 'help available' for more information.");
				return;
			}
			
			if(tokens.length >= 2) //Partner asked for what was available specifically
			{
			
				//Get the salvage name
				var type = tokens[1];	//first parameter
				if(tokens.length == 3)
					type = type + " " +tokens[2]; //second parameter (e.g. "Hide" in "Gromnie Hide")
					
				//Get a reference the GSSalvage object
				var salvage = GSSalvageManager.getSalvage(type);				
				if(salvage == null || salvage.totalAvailable() == 0)
				{
					GSTellPartner("None available. Please try back later, or check the spelling!");
					return;
				}
				
				var str = "The following are available:";
				
				for(var i=1; i!=11; i++)
				{
					var nr = salvage.getAvailable(i);
					if(nr > 0)
						str += " wk " + i + " (" + nr +" bag" +(nr>1?"s":"")+ "),";
				}
				str = str.substring(0, str.length -1);
				GSTellPartner(str);
				
			}
			else
			{
				//Create a comma delimited list of salvages that are available
				var s = "";			
				for(var k in GSSalvageManager.salvage)
					s += GSSalvageManager.salvage[k].getType() +", ";
						
				s = s.substring(0, s.length-2);
				GSTellPartner(s);
				return;
			}
			break;
			
		case "buy":
			if(Self.state == STATE_SUBTOTALING)
			{
				var total = Self.cart.getTotal();
				var pts = GSComputePointTotal();
				
				
				if(total > pts)
				{
					GSTellPartner("Hmm, you've provided " +pts+" points against a purchase of " +total+". Add " + (total - pts) +" more points and then we'll talk.");
					return;
				}
				
				Self.state = STATE_TRADING;
				skapi.TradeAccept();
				GSTellPartner("Please click 'Trade' to finish this!");
			}
			else
			{
				GSTellPartner("When you've added all the salvage you want, please tell me 'checkout' first and then add payment.");
			}
			break;
			
		case "clear":
			//Empty the shopping cart
			Self.cart.clear();
			GSTellPartner("Your shopping cart has been cleared.");
			
			//If we were showing the shopping cart, we should clear it now.
			if(Self.state == STATE_SUBTOTALING || Self.state == STATE_TRADING)
			{
				skapi.TradeReset();
				Self.state = STATE_DISPLAYING;
			}
			
			break;
			
		case "point":
		case "points":
			//Build list of strings with what is useful, and send to partner
			var msgs = new Array();
			var pts = GSGetPointList();
			msgs.push("Here is what I find useful:");
			for(var name in pts)
			{
				var value = pts[name];
				msgs.push(name + " = " + value);
			}
			GSMultitell(Self.partner, msgs);
			break;
			
		//Answer the 'check' word (TradeIt!) because some people don't read that this isn't
		//a normal salvage bot...
		case "check":
			GSTellPartner("This isn't a normal trade bot -- you don't need to check prices on individual bags.");
			GSTellPartner("Use 'price <salvage name>' to see the prices, for example: \"price iron\"");
			break;
			
		case "show": //show
			if(Self.state == STATE_DISPLAYING)
				GSShowItemsInTrade();
			else
				GSTellPartner("You must first complete the trade or clear the shopping cart!");
			break;
			
		case "help": //help <command>
			if(tokens.length == 1)
				GSMultitell(Self.partner, GS_HELP);
			else if(tokens.length == 2)
			{
				var topic = tokens[1].toLowerCase();
				
				for(var key in GS_HELPTOPICS)
				{
					var entry = GS_HELPTOPICS[key];
					
					if(entry[0] == topic)
					{
						GSMultitell(Self.partner, entry[1]);
						return;
					}
				}
				
				GSTellPartner("I couldn't find any help topics with that name.");
				
			}
			else //Too many parameters
				GSTellPartner("Please ask for help on just one subject at a time.");
		
			break;
		default:
			GSTellPartner("I didn't understand that! Please tell me 'help' for list of valid commands!");
	}
	
}

function GS_OnTradeAccept(aco)
{
	//Is the partner accepting the trade right now?
	if(aco.oid == Self.partner.oid)
	{
		if(Self.state == STATE_DISPLAYING)
			GSTellPartner("When you are ready to checkout, tell me 'checkout'");
		else if(Self.state == STATE_SUBTOTALING)
			GSTellPartner("When you have added enough payment, tell me 'buy'");
	}
}

function GS_OnTradeAdd(aco, side)
{
	if(side == 1) //Us, that is. Ignore this
		return;
	else
	{
		if(Self.state != STATE_SUBTOTALING)
			GSTellPartner("Please tell me 'checkout' before adding payment.");
		else //We are expecting payment
		{
			//Look up this item and see what its value is
			var pts = GSGetPoints(aco);
			if(pts <= 0)
			{
				GSTellPartner("The item you have added isn't one I accept. Please tell me 'checkout' again and add the proper items.");
				GSTellPartner("You can use 'points' to see what I accept as payment.");
				skapi.TradeReset();
				Self.state = STATE_DISPLAYING;
				return;
			}
			
			var total = Self.cart.getTotal();
			var pts = GSComputePointTotal();
			
			GSTellPartner("You've applied " + pts + " point(s) against a purchase of " + total + " point(s).");
			if(total >= pts)
				GSTellPartner("Tell me 'buy' to complete the trade!");
		}
	}
}

function GS_OnTradeEnd(arc)
{
	//Successful trades -> send them a thank you message
	if(arc == arcSuccess)
	{
		if(Self.partner != null)
			skapi.Tell(Self.partner.szName, "Thanks! This trade has been recorded");
			
		GSMessage("Trade completed!");
		Self.state = STATE_DISPLAYING;
		
		//Reindex inventory again.
		GSSalvageManager.rescan();
	}
	else
	{
		//Tell the person the closing statements
		if(Self.partner != null)
			GSMultitell(Self.partner, GSGetClosingText());
		
		GSMessage("Trade canceled.");
		Self.state = STATE_INACTIVE;	
		Self.partner = null;
		
		//Do we need to reindex our salvage? (occurs when bot is being restocked)
		if(Self.reindex == true)
		{
			GSSalvageManager.rescan();
			Self.reindex = false;
		}
		
	}
}

function GS_OnTradeReset(aco)
{
	//For some reason, when completing a trade, I get OnTradeReset() with aco == null
	if(aco == null)
	{
		//Just call the function...
		GS_OnTradeEnd(arcSuccess);
		return;
	}
	
	//Did the trading partner clear the trade window?
	if(aco.oid == Self.partner.oid)
	{
		skapi.Tell(Self.partner.szName, "You've cleared the trade window, tell me 'show' to show the list of items");
		Self.state = STATE_DISPLAYING;
	}
}

function GS_OnTradeStart(acoSender, acoReceiver)
{
	//Yay, we started a trade!
	Self.partner = acoSender;
	Self.state = STATE_DISPLAYING;
	Self.cart.clear();

	if(Self.reindex == true)
	{
		GSSalvageManager.rescan();
		Self.reindex = false;
	}
	
	//Greet the customer
	skapi.Tell(acoSender.szName, "GarageSale Bot Software v" +GS_VERSION + " by Buffslave II of Darktide");
	GSMultitell(Self.partner, GSGetGreetingText());
	
	GSShowItemsInTrade();
}