//GSPrices.js -- Prices and points
//Written by Buffslave II of Darktide
//Part of GarageSale Bot, under LGPL license

Prices = new Object();
Points = new Object();

/**
	@brief Gets the price of salvage given a type and workmanship
	@param type The type of salvage, as a string, such as "Iron"
	@param wk The workmanship of the salvage, as an integer, such as 7
	@return The price of the salvage
*/
function GSGetPrice(type, wk)
{
	type = type.toLowerCase();

	//If no explicit price structure was found, use the default entry.
	if(Prices[type] == null)
		type = "others";
	
	return Prices[type][wk-1]; //zero indexing, so index by -1
}

/**
	@brief Constructs a string representing the
	@param type The type of salvage, as a string, such as "Iron"
	@param workmin The minimum workmanship
	@param workmax The maximum workmanship
	@return The price string, presented to customers.
*/
function GSGetPriceString(type, workmin, workmax)
{
	var str = "";
	var price_entry = Prices[type];
	
	if(price_entry == null)
		price_entry = Prices["others"];
	
	for(var i=workmin; i<=workmax; i++)                    /* Pluralize ----v */
		str += "wk " + i + " (" + price_entry[i-1] +" pt" +(price_entry[i-1] > 1? "s" : "") + "), ";
	
	//Since we appended "), " to the end, we want to remove the comma and space so it
	//doesn't look like we have more following
	str = str.substring(0, str.length-2);
	return str;
}

function GSGetPoints(aco)
{
	if(Points[aco.szName] == null)
		return 0;
	
	return Points[aco.szName] * aco.citemStack;
}

function GSGetPointList()
{
	return Points;
}

function GSInitializePrices()
{
	var fso, fp;
	var line_number;
	fso = new ActiveXObject("Scripting.FileSystemObject");
	fp = fso.OpenTextFile("prices.txt", 1); // 1 <-> read only

	GSMessage("Loading prices from prices.txt");
	
	line_number = 0;
	while(!fp.AtEndOfStream)
	{
		var line = fp.ReadLine();
		var entry = new Array();
		line_number++;

		//Ignore lines that begin with
		if(line.substr(0,2) == "//")
			continue;
			
		//Empty line?
		if(line.replace(/\s/, "").length == 0)
			continue;
			
		//Split line by whitespace
		var fields = line.split(/\s/); 

		//Replace underscores by spaces for salvage name
		fields[0] = fields[0].replace("_", " ");
		
		if(fields.length < 11)
		{
			GSError("prices.txt line " + line_number + ": need 10 prices, found "+ (fields.length-1));
			continue;
		}
		
		Prices[fields[0]] = entry;

		for(var i=1; i<=10; i++)
			entry[i-1] = parseInt(fields[i]);
		
	}
	fp.Close();
	fp = null;
	
	fp = fso.OpenTextFile("points.txt", 1); // 1 <-> read only
	GSMessage("Loading points from points.txt");
	line_number = 0;
	while(!fp.AtEndOfStream)
	{
		var line = fp.ReadLine();
		line_number++;

		//Ignore lines that begin with
		if(line.substr(0,2) == "//")
			continue;
			
		//Empty line?
		if(line.replace(/\s/, "").length == 0)
			continue;
		
		//Split line by whitespace
		var fields = line.split(/\s/); 
		
		if(fields.length < 2)
		{
			GSError("points.txt line " + line_number + ": need a point value after "+fields[0]);
			continue;
		}
		
		//Combine mulitpart names (e.g "Trade Note (250,000)") into one string
		var name = "";
		for(var i=0; i<fields.length-1; i++)
			name += fields[i] + " ";
		name = name.substring(0, name.length-1);
		
		//Save point value now
		Points[name] = parseInt(fields[fields.length-1]);
	}
	fp.Close();
	fp = null;	
}
