
//Constructor for GSSalvage class
function GSSalvage(salvage_name)
{
	var i;
	
	this.name = salvage_name;
	this.available = new Array();
	this.oids = new Array(); //Array of arrays to hold OIDs for each bag
	
	//Initially, no salvage of this type is available (all set to 0)
	for(i=1; i != 11; i++)
	{
		this.available[i] = 0;
		this.oids[i] = new Array();
	}
	
}

GSSalvage.prototype.getType = function()
{
	return this.name;
}

//Get the number of bags of salvage available, given an integral worksmanship
GSSalvage.prototype.getAvailable = function(work)
{
	return this.available[work];
}

//Increments the number of bags of salvage available, given the integral worksmanship
GSSalvage.prototype.addAvailable = function(aco)
{
	var work = Math.floor(aco.workmanship);
	
	this.available[work] += 1;
	this.oids[work].push(aco.oid)
}

GSSalvage.prototype.getOIDs = function(work)
{
	return this.oids[work];
}

GSSalvage.prototype.hasAvailable = function(workmin, workmax, nr_bags)
{
	//Stupid question
	if(nr_bags <= 0)
		return false;
	for(var i = workmin; i<=workmax; i++)
	{
		nr_bags -= this.available[i];
	}
	
	//Were we able to satisfy this request?
	if(nr_bags <= 0)
		return true;


	return false;
}

//Gets the total number of bags
GSSalvage.prototype.totalAvailable = function()
{
	var i;
	var sum = 0;
	for(i=1; i != 11; i++)
		sum += this.available[i];
	
	return sum;
}

function GSParseWork(workstr)
{
	//Strings are either in the format of "[x,y]" or "x+" with the "+" optional
	var wkmin = 0;
	var wkmax = 0;
	
	if(workstr == null)
		return null;
	
	//Case of [x,y]
	if(workstr.charAt(0) == '[')
	{
		//Split into "[x" and "y]"
		var parts = workstr.split(",");
		
		//Make sure we have two parts
		if(parts.length != 2)
			return null;
		
		//Now strip []'s from the strings and parse the integer values from them.
		wkmin = parseInt(parts[0].replace("[",""));
		wkmax = parseInt(parts[1].replace("]",""));
	}
	else
	{
		//Was there no "+"?
		if(workstr.indexOf("+") < 0)
			wkmin = wkmax = parseInt(workstr); //Then they only want a single workmanship
		else
		{
			//"x+" implies [x,10]
			wkmin = parseInt(workstr.replace("+",""));
			wkmax = 10;
		}
	}
	
	//Catch invalid values here.
	if(wkmin <= 0 || wkmax <= 0 || wkmin > wkmax || wkmin > 10 || wkmax > 10)
		return null;

		
	
	var retval = new Object();
	retval.work_min = wkmin;
	retval.work_max = wkmax;
	return retval;
}